#include "Socket.h"

bool Socket::LoadWSA(WSADATA& ws){
	if (WSAStartup(0x0202, &ws))
	{
		return 0;
	}
	return 1;
}
bool Socket::UnLoadWSA(){
	if (WSACleanup() == SOCKET_ERROR){
		return 0;
	}
	return 1;
}
Socket::Socket(void)
{
	this->socket = NULL;
	memset(&this->addr, 0, sizeof(addr));
	this->err = 0;
	this->size_send = 0, this->size_recv = 0;
}
Socket::~Socket(void)
{
}
bool Socket::CreateSocket(void){


	this->socket = ::socket(AF_INET, SOCK_STREAM, IPPROTO_IP);

	if (this->socket == SOCKET_ERROR){

		this->err = WSAGetLastError();
		return 0;
	}

	return 1;


}
bool Socket::SendData(LPCSTR _pbuf, const int & size_buf){

	this->size_send = send(this->socket, _pbuf, size_buf, 0);

	if (this->size_send == SOCKET_ERROR){
		this->err = WSAGetLastError();
		return 0;
	}
	return 1;


}
bool Socket::RecvData(LPSTR _pbuf, const int & size_buf){


	this->size_recv = recv(this->socket, _pbuf, size_buf, 0);

	if (this->size_recv == SOCKET_ERROR){

		this->err = WSAGetLastError();
		return 0;
	}
	return 1;


}
bool Socket::SendDataAll(std::string& data ){

	int ReadWrite = 0;
	while (ReadWrite < data.length())
	{
		if(!this->SendData(&data[0+ReadWrite],(data.length()-ReadWrite) )){
			return 0;
		}
		ReadWrite += this->size_send;
	}
	return 1;

}

bool Socket::RecvDataAll(std::string& data ){

	int SizeRequest = 1024;
	int ReadWrite = 0;

	char * RecvData = (char*)malloc(SizeRequest);

	while(true){
		ZeroMemory(RecvData,SizeRequest);
		if(!this->RecvData(RecvData,SizeRequest-1)){
			return 0;
		}
		if(this->size_recv == 0 || this->size_recv == SOCKET_ERROR){
			break;	
		}
		data += RecvData;
	}
	free(RecvData);

	return 1;
}
bool Socket::CloseSocket(void){
	this->ShootDown(2);
	if (closesocket(this->socket) == SOCKET_ERROR){
		this->err = WSAGetLastError();
		return false;
	}
	return 1;
}
int Socket::ShootDown(const int& i){
	return shutdown(this->socket,i);
}
int Socket::Select(const int& i,const int& sec){

	fd_set read_s; // Множество
	timeval time_out; // Таймаут

	FD_ZERO (&read_s); // Обнуляем множество
	FD_SET (this->socket, &read_s); // Заносим в него наш сокет 
	time_out.tv_sec = sec;time_out.tv_usec = 250000; //Таймаут 0.5 секунды.


	if(i == 0){
		return select(0,&read_s, NULL, NULL, &time_out);
	}else if( i == 1){
		return select(0,NULL, &read_s, NULL, &time_out);
	}else{
		return select(0,NULL, NULL, &read_s, &time_out);
	}

}
int Socket::GetError(void)const{
	return this->err;
}
int Socket::GetSendSize(void)const{
	return this->size_send;

}
int Socket::GetRecvSize(void)const{
	return this->size_recv;
}


//================================================================================

// Подключение к серверу
bool Socket::Connect(	LPCSTR		_ip,		// IP/Name сервера
					 const UINT	&_port )	// Порт сервера
{


	// Заполнение структуры сетевой информации о сервере
	this->addr.sin_addr.s_addr = inet_addr( _ip );
	if( INADDR_NONE == this->addr.sin_addr.s_addr )
	{
		struct hostent *h = gethostbyname( _ip );
		if( !h )
		{
			this->err = WSAGetLastError( );
			return false;
		}
		this->addr.sin_addr = *(struct in_addr *) h->h_addr_list[0];
	}
	this->addr.sin_family	= AF_INET;
	this->addr.sin_port		= htons( _port );

	// Соединение с сервером
	if( connect( this->socket, ( const struct sockaddr* )&this->addr, sizeof( this->addr ) ) == SOCKET_ERROR )
	{
		this->err = WSAGetLastError( );
		return false;
	}

	return true;
}
bool Socket::Disconnect( void )
{

	this->CloseSocket();
	if(this->CreateSocket()){
		return 1;
	}

	return 0;
}




//================================================================================

bool Socket::Bind(LPCSTR _ip , const UINT &_port, const int	&_count_client){

	this->addr.sin_family		= AF_INET;
	this->addr.sin_port			= htons( _port );
	this->addr.sin_addr.s_addr	= inet_addr( _ip );


	if( bind( this->socket, ( const sockaddr* ) &this->addr, sizeof( this->addr ) ) == SOCKET_ERROR )

	{
		this->err = WSAGetLastError( );
		return false;
	}else{
		return true;
	}

}

bool Socket::Listen(void){

	if ( listen( this->socket, SOMAXCONN ) == SOCKET_ERROR )
	{
		this->err = ::WSAGetLastError( );
		return false;
	}else{
		return true;
	}

}


bool Socket::Accept(Socket &_client){

	SOCKET client_socket	= NULL;    
	sockaddr_in client_addr = { 0 };		
	int client_addr_size	= sizeof( client_addr );

	client_socket			= accept(	this->socket, 
		( sockaddr* )&client_addr, 
		&client_addr_size );

	if( client_socket == SOCKET_ERROR )
	{
		this->err = WSAGetLastError( );
		return false;
	}

	_client.SaveSocketAndSockaddr( client_socket, &client_addr );
	return true;
}

// Метод установки сохранение сокета и структуры сетевой информации
void Socket::SaveSocketAndSockaddr( const SOCKET		&_socket,	// Сокет
								   const sockaddr_in	*_paddr )	// Структура sockaddr_in
{
	this->socket = _socket;
	memcpy_s( &this->addr, sizeof( sockaddr_in ), _paddr, sizeof( sockaddr_in ) );
}

//===================================================================================